<?php
/*
 * Gpg Helper Plugin.
 *
 * @license  Apache 2 (http://www.apache.org/licenses/LICENSE-2.0)
 * @author   Kazuyuki Matsuda <kazm@shorindo.com>
 */

if (!defined('DOKU_INC')) die();
if (!defined('DOKU_LF')) define('DOKU_LF', "\n");
if (!defined('DOKU_TAB')) define('DOKU_TAB', "\t");

class helper_plugin_gpg extends DokuWiki_Plugin {
    function getInfo(){
        return confToHash(dirname(__FILE__).'/plugin.info.txt');
    }

    function getFormat(){
        return 'gpg';
    }

    function canRender($format) {
        return ($format == 'gpg');
    }

    function getMethods() {
        return array();
    }

    function plugin($name, $data){
    }

    function encrypt($text, $pass) {
        //dbglog("encrypt({$text}, {$pass})");
        $program = $this->getConf('program').
            " -ca --homedir /tmp --no-mdc-warning --batch --passphrase-fd 3";
        $descriptorspec = array(
            0 => array("pipe", "r"),
            1 => array("pipe", "w"),
            2 => array("pipe", "w"),
            3 => array("pipe", "r")
            );
        $process = proc_open($program, $descriptorspec, $pipes);
        if (is_resource($process)) {
            fwrite($pipes[0], $text);
            fclose($pipes[0]);
            fwrite($pipes[3], $pass);
            fclose($pipes[3]);
            $stdout = stream_get_contents($pipes[1]);
            fclose($pipes[1]);
            $stderr .= stream_get_contents($pipes[2]);
            fclose($pipes[2]);
            $ret = proc_close($process);
            if ($ret != 0)
                return "NG {$ret}\n".$stderr;
            else
                return "OK 0\n".$stdout;
        }
    }

    function decrypt($text, $pass) {
        //dbglog("decrypt({$text}, {$pass})");
        $program = $this->getConf('program').
            " --homedir /tmp --no-mdc-warning --batch --passphrase-fd 3";
        $descriptorspec = array(
            0 => array("pipe", "r"),
            1 => array("pipe", "w"),
            2 => array("pipe", "w"),
            3 => array("pipe", "r")
            );
        $process = proc_open($program, $descriptorspec, $pipes);
        if (is_resource($process)) {
            fwrite($pipes[0], $text);
            fclose($pipes[0]);
            fwrite($pipes[3], $pass);
            fclose($pipes[3]);
            $stdout = stream_get_contents($pipes[1]);
            fclose($pipes[1]);
            $stderr .= stream_get_contents($pipes[2]);
            fclose($pipes[2]);
            $ret = proc_close($process);
            if ($ret != 0)
                return "NG {$ret}\n".$stderr;
            else
                return "OK 0\n".$stdout;
        }
    }

    function decrypt_html($id, $pass) {
        $id = cleanID($id);
        if (auth_quickaclcheck($id) < AUTH_READ) {
            return
                "NG 1\n".
                "Permission denied.";
        }

        $fn = wikiFN($id);
        $text = preg_replace('/<gpg>(.*?)<\/gpg>/', '$1', file_get_contents($fn));
        $lines = preg_split("/\n/", $this->decrypt($text, $pass));
        if (preg_match('/^NG \d+/', $lines[0])) {
            return implode("\n", $lines);
        }
        array_shift($lines);
        $wiki = implode("\n", $lines);
        return
            "OK 0\n".
            p_render('xhtml', p_get_instructions($wiki), $info);
    }
}
