/**
 * Gpg plugin script.
 *
 * @license  Apache 2 (http://www.apache.org/licenses/LICENSE-2.0)
 * @author   Kazuyuki Matsuda <kazm@shorindo.com>
 */
var plugin_gpg = {
	init : function() {
		var textarea = $('wiki__text');
		if (!textarea) return;
		if (!textarea.value.match(/^\s*<gpg>[\s\S]*?<\/gpg>\s*$/m)) return;
		plugin_gpg.dialog(LANG['plugins']['gpg']['confirm_decrypt'],
			function() {
				var text = $('wiki__text').value
					.replace(/<gpg>([\s\S]*?)<\/gpg>/m, '$1');
				var pass = $('gpg__passphrase').value;
				plugin_gpg.decrypt(text, pass);
			},
			function() {
				location = location.href;
			});
	},
	finish : function() {
		var node = $('gpg__dialog');
		if (node) node.parentNode.removeChild(node);
		node = $('gpg__background');
		if (node) node.parentNode.removeChild(node);
	},
	getSize : function (node) {
		return {
			dwidth : node.clientWidth,
			dheight : node.clientHeight,
			width : node.scrollWidth,
			height : node.scrollHeight
		};
	},
	dialog : function(message, ok_action, cancel_action) {
		var FRAME_COLOR = '#F0F0F0';
		var BG_COLOR = '#FFFFFF';
		var bg = plugin_gpg.showbackground();
		var divdoku = getElementsByClass('dokuwiki', document.body, 'div')[0];
		var frame = divdoku.appendChild(document.createElement("div"));
		frame.id = 'gpg__dialog';

		var head = frame.appendChild(document.createElement("div"));
		head.className = head.styleClass = "gpg_dialog_head";
		head.innerHTML = 'パスワード入力';

		var body = frame.appendChild(document.createElement("div"));
		body.className = head.styleClass = "gpg_dialog_body";
		body.innerHTML = 
			'<div id="gpg__message">' +
			message +
			'</div>' +
			'<div style="text-align:center;">' +
			'<input type="password" id="gpg__passphrase" class="edit" name="passphrase" style="width:100px;"/>' +
			'</div>';

		var foot = frame.appendChild(document.createElement("div"));
		foot.className = foot.styleClass = "gpg_dialog_foot";
		var button = document.createElement("input");
		button.type = 'button';
		foot.appendChild(button);
		button.id = "gpg__okbutton";
		button.className = button.styleClass = "edit";
		button.value = "ＯＫ";
		button.onclick = ok_action;

		button = document.createElement("input");
		button.type = 'button';
		button = foot.appendChild(button);
		button.id = "gpg__cancelbutton";
		button.className = button.styleClass = "edit";
		button.value = "キャンセル";
		button.onclick = cancel_action;
 
		if (frame.style.setExpression) { //IE
			var wsize = plugin_gpg.getSize(document.documentElement);
			frame.style.position = "absolute";
			frame.dy = (wsize.dheight - frame.offsetHeight) / 2;
			frame.dx = (wsize.dwidth - frame.offsetWidth) / 2;
			frame.style.setExpression('top', 'y=(document.documentElement.scrollTop+this.dy)+"px"');
			frame.style.setExpression('left', 'x=(document.documentElement.scrollLeft+this.dx)+"px"');
		} else {
			frame.style.position = "fixed";
			frame.style.left = "50%";
			frame.style.top = "50%";
			frame.style.marginLeft = (-frame.offsetWidth / 2) + "px";
			frame.style.marginTop = (-frame.offsetHeight / 2) + "px";
		}

		$('gpg__passphrase').focus();
	},
	showbackground : function() {
		var wsize = plugin_gpg.getSize(document.documentElement);
		var divdoku = getElementsByClass('dokuwiki', document.body, 'div')[0];
		var frame = divdoku.appendChild(document.createElement("div"));
		//var frame = document.body.appendChild(document.createElement("div"));
		frame.id = 'gpg__background';
		if (frame.style.setExpression) { //IE
			frame.style.position = "absolute";
			frame.style.top = "0px";
			frame.style.left = "0px";
			frame.style.setExpression('top', 'y=(document.documentElement.scrollTop)+"px"');
			frame.style.setExpression('left', 'x=(document.documentElement.scrollLeft)+"px"');
		} else {
			frame.style.position = "fixed";
		}
		frame.style.left = "0px";
		frame.style.top = "0px";
		frame.style.width = wsize.dwidth + "px";
		frame.style.height = wsize.dheight + "px";
		frame.style.backgroundColor = "gray";
		frame.style.opacity = 0.5;
		frame.style.filter = 'Alpha(opacity=50)';
		window.onresize = function() {
			var wsize = plugin_gpg.getSize(document.documentElement);
			frame.style.width = wsize.dwidth + "px";
			frame.style.height = wsize.dheight + "px";
		}
		return frame;
	},
	createhttp : function() {
		var xmlhttp;
		try {
			xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
		} catch (e) {
			try {
				xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
			} catch (err) {
				xmlhttp = null;
			};
		};
		if(!xmlhttp && typeof XMLHttpRequest != "undefined"){
			xmlhttp = new XMLHttpRequest();
		}
		return xmlhttp;
	},

	error : function(message) {
		var node = $('gpg__message');
		if (node) {
			node.innerHTML = message;
		}
	},

	encrypt : function(text, pass) {
		var xmlhttp = this.xmlhttp = plugin_gpg.createhttp();
		var self = plugin_gpg;
		xmlhttp.open("POST", DOKU_BASE + 'lib/exe/ajax.php', true);
		xmlhttp.onreadystatechange = function() {
			self.encrypt_onload.apply(self);
		};
		xmlhttp.send('call=gpg&cmd=encrypt' +
			'&text=' + encodeURI(text).replace(/\+/g,'%2B') +
			'&pass=' + encodeURI(pass).replace(/\+/g,'%2B'));
	},

	encrypt_onload : function() {
		if (this.xmlhttp.readyState != 4)
			return;
		var response = this.xmlhttp.responseText.split("\n");
		if (this.xmlhttp.status != 200) {
			alert(this.xmlhttp.responseText);
			return;
		}

		var textarea = $('wiki__text');
		var status = response.shift();
		if (status.match(/^NG/)) {
			alert(response.join("\n"));
		} else if (textarea) {
			textarea.value = "<gpg>\n"+response.join("\n")+"</gpg>";
			textarea.readOnly = true;
			plugin_gpg.finish();
		}
	},

	decrypt : function(text, pass) {
		var xmlhttp = this.xmlhttp = plugin_gpg.createhttp();
		var self = this;
		xmlhttp.open("POST", DOKU_BASE + 'lib/exe/ajax.php', true);
		xmlhttp.onreadystatechange = function() {
			self.decrypt_onload.apply(self);
		};
		xmlhttp.send('call=gpg&cmd=decrypt' +
			'&text=' + encodeURI(text).replace(/\+/g,'%2B') +
			'&pass=' + encodeURI(pass).replace(/\+/g,'%2B'));
	},

	decrypt_onload : function() {
		if (this.xmlhttp.readyState != 4)
			return;
		var response = this.xmlhttp.responseText.split("\n");
		if (this.xmlhttp.status != 200) {
			alert(this.xmlhttp.responseText);
			return;
		}

		var textarea = $('wiki__text');
		var status = response.shift();
		if (status.match(/^NG/)) {
			plugin_gpg.error(response.join('<br/>'));
		} else if (textarea) {
			textarea.value = response.join("\n");
			textarea.readOnly = false;
			plugin_gpg.finish();
		}
	},

	decrypt_html : function(form) {
		var pass = form.elements['pass'].value;
		var xmlhttp = this.xmlhttp = plugin_gpg.createhttp();
		var self = plugin_gpg;
		xmlhttp.open("POST", DOKU_BASE + 'lib/exe/ajax.php', true);
		xmlhttp.onreadystatechange = function() {
			self.decrypt_html_onload.apply(self);
		};
		xmlhttp.send('call=gpg&cmd=decrypt_html' +
			'&id=' + encodeURI(JSINFO['id']) +
			'&pass=' + encodeURI(pass).replace(/\+/g,'%2B'));

	},

	decrypt_html_onload : function() {
		if (this.xmlhttp.readyState != 4)
			return;
		var response = this.xmlhttp.responseText.split("\n");
		if (this.xmlhttp.status != 200) {
			alert(this.xmlhttp.responseText);
			return;
		}

		var body = $('gpg__body');
		var status = response.shift();
		if (status.match(/^NG/)) {
			$('gpg__message').innerHTML = response.join('<br/>');
		} else {
			body.innerHTML = response.join("\n");
		}
	},

	dialog_start_encrypt : function() {
		plugin_gpg.dialog(LANG['plugins']['gpg']['start_encrypt'],
			function() { //OK
				var pass = $('gpg__passphrase').value;
				if (pass == null || pass.length < 4) {
					plugin_gpg.error(LANG['plugins']['gpg']['empty_passphrase']);
					return;
				}
				plugin_gpg.finish();
				plugin_gpg.dialog_confirm_encrypt(pass);
			},
			function() { //CANCEL
				plugin_gpg.finish();
			});
	},

	dialog_confirm_encrypt : function(passphrase) {
		//confirm and encrypt
		plugin_gpg.dialog(LANG['plugins']['gpg']['confirm_encrypt'],
			function() { //OK
				var text = $('wiki__text').value;
				var pass = $('gpg__passphrase').value;
				if (passphrase != pass) {
					plugin_gpg.error(LANG['plugins']['gpg']['invalid_passphrase']);
					return;
				}
				plugin_gpg.encrypt(text, pass);
			},
			function() { //CANCEL
				plugin_gpg.finish();
			});
	},

	dialog_decrypt : function() {
		plugin_gpg.dialog(LANG['plugins']['gpg']['confirm_decrypt'],
			function() {
				var text = $('wiki__text').value
					.replace(/<gpg>([\s\S]*?)<\/gpg>/m, '$1');
				var pass = $('gpg__passphrase').value;
				plugin_gpg.decrypt(text, pass);
				},
			function() {
				plugin_gpg.finish();
			});
	}
};
function addBtnActionGpg(btn, props, edid) {
	addEvent(btn,'click',function(){
		var text = $('wiki__text').value;
		if (text.replace(/^\s*<gpg>[\s\S]*<\/gpg>\s*$/mg, '')) {
			plugin_gpg.dialog_start_encrypt();
		} else {
			plugin_gpg.dialog_decrypt();
		}
		return false;
	});
	return true;
};
addInitEvent(plugin_gpg.init);
